/*
 * LIRC plugin for the Xbox DVD playback kit remote control
 * key codes are obtained from the xpad driver
 *
 *   Marko Friedemann <mfr@bmx-chemnitz.de>
 *
 * this is loosely based on lirc_gpio.c:
 * (L) by Artur Lipowski <alipowski@kki.net.pl>
 *     patch for the AverMedia by Santiago Garcia Mantinan <manty@i.am>
 *                            and Christoph Bartelmus <lirc@bartelmus.de>
 *     patch for the BestBuy by Miguel Angel Alvarez <maacruz@navegalia.com>
 *     patch for the Winfast TV2000 by Juan Toledo <toledo@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: lirc_xir.c,v 1.16 2002/08/30 11:57:28 lirc Exp $
 *
 */

#include <linux/version.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 2, 4)
#error "*******************************************************"
#error "Sorry, this driver needs kernel version 2.2.4 or higher"
#error "*******************************************************"
#endif

#include <linux/module.h>
#include <linux/kmod.h>
#include <linux/sched.h>
#include <linux/wrapper.h>
#include <linux/errno.h>

#include <lirc_dev.h>
#include <xpad.h>

static int debug = 0;
static int minor = -1;

MODULE_PARM(debug,"i");

#define dprintk  if (debug) printk

#define MAX_BYTES 8

#define SUCCESS 0
#define LOGHEAD "lirc_xir: "

/*
 * this simply calls the get_keycode function of xpad_ir and
 * gets a byte containing the keycode of the key pressed back
 */
static int get_key(void* data, unsigned char *key, int key_no)
{
	unsigned char _key;
	
	if ((_key = xpad_ir_get_keycode()) == 0)
	    return -EAGAIN;
	    
	dprintk(LOGHEAD "get_key called: %d\n", _key);
	*key = _key;
	
	return SUCCESS;
}

/* tell xpad_ir to begin USB communication (if needed) */
static void set_use_inc(void* data)
{
	MOD_INC_USE_COUNT;
	xpad_ir_lirc_open();
}

/* tell xpad_ir to stop USB communication (if not open elsewhere) */
static void set_use_dec(void* data)
{
	xpad_ir_lirc_close();
	MOD_DEC_USE_COUNT;
}

/*static wait_queue_head_t* get_queue(void* data)
{
}*/

/*
 * cyclic check (33 ms)
 * 
 * maybe implement the get_queue feature so that
 *  we can tell lirc when some key was pressed
 */
static struct lirc_plugin plugin = {
	"lirc_xir  ",
	-1,
	8,	/* 1 byte per key */
	33,	/* cyclic check */
	NULL,
	get_key,
	NULL/*get_queue*/,
	set_use_inc,
	set_use_dec
};

EXPORT_NO_SYMBOLS; 

/* Dont try to use it as a static version !  */

#ifdef MODULE
MODULE_DESCRIPTION("LIRC plugin module for Xbox DVD remote");
MODULE_AUTHOR("Marko Friedemann");
#ifdef MODULE_LICENSE
MODULE_LICENSE("GPL");
#endif

/*
 *
 */
int init_module(void)
{
	minor = lirc_register_plugin(&plugin);

	if (0 > minor) {
		printk (LOGHEAD "device registration failed with %d\n", minor);
		return minor;
	}

	printk(LOGHEAD "driver registered (minor %d)\n", minor);

	return SUCCESS;
}

/*
 *
 */
void cleanup_module(void)
{
	int ret;

	ret = lirc_unregister_plugin(minor);
 
	if (0 > ret) {
		printk(LOGHEAD "error in lirc_unregister_plugin (%d): %d\n"
		       "Trying again...\n",
		       minor, ret);

		current->state = TASK_INTERRUPTIBLE;
		schedule_timeout(HZ);

		ret = lirc_unregister_plugin(minor);
 
		if (0 > ret) {
			printk(LOGHEAD "error in lirc_unregister_plugin (%d): %d!!!\n",
			       minor, ret);
			return;
		}
	}

	dprintk(LOGHEAD "plugin (%d) successfully unloaded\n", minor);
}
#endif

/*
 * Overrides for Emacs so that we follow Linus's tabbing style.
 * ---------------------------------------------------------------------------
 * Local variables:
 * c-basic-offset: 8
 * End:
 */
